// ======================================================================
//
// Appear.h
// jeff grills
//
// copyright 1996 origin systems inc
//
// ======================================================================

#ifndef APPEAR_H
#define APPEAR_H

// ======================================================================

class Camera;
class Extents;
class HardPoints;
class Iff;
class Object;
class Xform;

#include "Appear.def"
#include "CrcString.h"

// ======================================================================

class Appear
{
	friend class Appear;

private:

	static Appear  *first;

protected:

	AppearId       id;
	int            users;
	int            token;
	real           scale;
	CrcString     *crcName;
	char          *name;
	Appear        *nextAppear;

	bool           needsRealAlter;

	Extents       *extents;
	HardPoints    *hardPoints;
	
protected:

	static bool    found(const Appear *a, const CrcString *crcname, real scale);
	static Appear *find(const char *name, Iff &iff, real scale);

protected:
	
	void           loadShared(Iff &iff, const char *id);
	virtual void   realAlter(real time);

private:

	// disable the copy constructor and assignment operators
	Appear(void);
	Appear(const Appear &);
	Appear &operator =(const Appear &);

public:

	static Appear *create(const char *name, Iff &iff, real scale);
	static Appear *find(const char *name, real scale, bool shouldAddUser=true);
	static Appear *addUser(Appear *appear);
	static void    removeUser(Appear *appear);

public:

	Appear(AppearId newId, const char *newName, real newScale);
	virtual ~Appear(void);

	AppearId        getId(void) const;

	// get the name of the appearance
	const char      *getName(void) const;
	const CrcString *getCrcName(void) const;

	// get the sort token
	int             getToken(void) const;

	// update the the appearance appropriately
	bool            alterRequired(void) const;
	void            alter(real time);

	// hardpoint information
	const HardPoints *getHardPoints(void) const;
	virtual void      setHardPoints(HardPoints *newHardPointsbb);

	// extent information
	const Extents *getExtents(void) const;
	virtual void   setExtents(Extents *newExtents);

	virtual const Appear *getRenderAppear(Camera &camera, const Object *object) const;
	virtual void          render(Camera &camera, const Xform &objectToWorld) const;
};

// ======================================================================

inline AppearId Appear::getId(void) const
{
	return id;
}

// ----------------------------------------------------------------------

inline int Appear::getToken(void) const
{
	return token;
}

// ----------------------------------------------------------------------

inline const char *Appear::getName(void) const
{
	return name;
}

// ----------------------------------------------------------------------

inline const CrcString *Appear::getCrcName(void) const
{
	return crcName;
}

// ----------------------------------------------------------------------

inline const Extents *Appear::getExtents(void) const
{
	return extents;
}

// ----------------------------------------------------------------------

inline const HardPoints *Appear::getHardPoints(void) const
{
	return hardPoints;
}

// ----------------------------------------------------------------------

inline bool Appear::alterRequired(void) const
{
	return needsRealAlter;
}

// ----------------------------------------------------------------------

inline void Appear::alter(real time)
{
	if (alterRequired())
		realAlter(time);
}

// ======================================================================

#endif
